import 'package:flutter/material.dart';
import 'package:yuro_core/yuro_core.dart';
import 'package:yuro_extension/yuro_extension.dart';
import 'package:yuro_route/yuro_route.dart';
import 'package:yuro_state/yuro_state.dart';
import 'package:yuro_translation/yuro_translation.dart';
import 'package:yuro_widget/yuro_widget.dart';

import 'yuro_app_controller.dart';
import 'yuro_app_ext.dart';

class YuroMaterialApp extends StatelessWidget {
  const YuroMaterialApp({
    Key key,
    this.onInit,
    this.onReady,
    this.onDispose,
    this.translations,
    this.fallbackLocale,
    this.yuroPages,
    this.unknownPage,
    this.customTransition,
    this.routingCallback,
    this.smartManagement = SmartManagement.full,
    //
    this.navigatorKey,
    this.home,
    this.routes,
    this.initialRoute,
    this.onGenerateRoute,
    this.onGenerateInitialRoutes,
    this.onUnknownRoute,
    this.navigatorObservers = const <NavigatorObserver>[],
    this.builder,
    this.title,
    this.onGenerateTitle,
    this.color,
    this.theme,
    this.darkTheme,
    this.highContrastTheme,
    this.highContrastDarkTheme,
    this.themeMode,
    this.locale,
    this.localizationsDelegates,
    this.localeListResolutionCallback,
    this.localeResolutionCallback,
    this.supportedLocales,
    this.debugShowMaterialGrid,
    this.showPerformanceOverlay,
    this.checkerboardRasterCacheImages,
    this.checkerboardOffscreenLayers,
    this.showSemanticsDebugger,
    this.debugShowCheckedModeBanner,
    this.shortcuts,
    this.actions,
  })  : assert(navigatorObservers != null),
        assert(unknownPage != null || onUnknownRoute != null),
        routeInformationProvider = null,
        routeInformationParser = null,
        routerDelegate = null,
        backButtonDispatcher = null,
        super(key: key);

  const YuroMaterialApp.router({
    Key key,
    this.onInit,
    this.onReady,
    this.onDispose,
    this.translations,
    this.fallbackLocale,
    this.smartManagement = SmartManagement.full,
    //
    this.routeInformationProvider,
    @required this.routeInformationParser,
    @required this.routerDelegate,
    this.backButtonDispatcher,
    this.builder,
    this.title,
    this.onGenerateTitle,
    this.color,
    this.theme,
    this.darkTheme,
    this.highContrastTheme,
    this.highContrastDarkTheme,
    this.themeMode,
    this.locale,
    this.localizationsDelegates,
    this.localeListResolutionCallback,
    this.localeResolutionCallback,
    this.supportedLocales,
    this.debugShowMaterialGrid,
    this.showPerformanceOverlay,
    this.checkerboardRasterCacheImages,
    this.checkerboardOffscreenLayers,
    this.showSemanticsDebugger,
    this.debugShowCheckedModeBanner,
    this.shortcuts,
    this.actions,
  })  : assert(routeInformationParser != null),
        assert(routerDelegate != null),
        navigatorKey = null,
        navigatorObservers = null,
        initialRoute = null,
        onGenerateInitialRoutes = null,
        home = null,
        routes = null,
        yuroPages = null,
        onGenerateRoute = null,
        unknownPage = null,
        onUnknownRoute = null,
        customTransition = null,
        routingCallback = null,
        super(key: key);

  final VoidCallback onInit;
  final VoidCallback onReady;
  final VoidCallback onDispose;
  final Map<String, Map<String, String>> translations;
  final Locale fallbackLocale;
  final List<YuroPage> yuroPages;
  final YuroPage unknownPage;
  final CustomTransition customTransition;
  final RoutingCallback routingCallback;
  final SmartManagement smartManagement;

  final GlobalKey<NavigatorState> navigatorKey;
  final Widget home;
  final Map<String, WidgetBuilder> routes;
  final String initialRoute;
  final RouteFactory onGenerateRoute;
  final InitialRouteListFactory onGenerateInitialRoutes;
  final RouteFactory onUnknownRoute;
  final List<NavigatorObserver> navigatorObservers;
  final RouteInformationProvider routeInformationProvider;
  final RouteInformationParser<Object> routeInformationParser;
  final RouterDelegate<Object> routerDelegate;
  final BackButtonDispatcher backButtonDispatcher;
  final TransitionBuilder builder;
  final String title;
  final GenerateAppTitle onGenerateTitle;
  final ThemeData theme;
  final ThemeData darkTheme;
  final ThemeData highContrastTheme;
  final ThemeData highContrastDarkTheme;
  final ThemeMode themeMode;
  final Color color;
  final Locale locale;
  final Iterable<LocalizationsDelegate<dynamic>> localizationsDelegates;
  final LocaleListResolutionCallback localeListResolutionCallback;
  final LocaleResolutionCallback localeResolutionCallback;
  final Iterable<Locale> supportedLocales;
  final bool showPerformanceOverlay;
  final bool checkerboardRasterCacheImages;
  final bool checkerboardOffscreenLayers;
  final bool showSemanticsDebugger;
  final bool debugShowCheckedModeBanner;
  final Map<LogicalKeySet, Intent> shortcuts;
  final Map<Type, Action<Intent>> actions;
  final bool debugShowMaterialGrid;

  List<Route<dynamic>> generateInitialRoutes(String name) =>
      [YuroPageRedirect(RouteSettings(name: name), unknownPage: unknownPage, onUnknownRoute: onUnknownRoute).route()];

  Route<dynamic> generateRoute(RouteSettings settings) =>
      YuroPageRedirect(settings, unknownPage: unknownPage, onUnknownRoute: onUnknownRoute).route();

  @override
  Widget build(BuildContext context) => YuroBuilder<YuroAppController>(
      init: Yuro.appController,
      initState: (_) {
        WidgetsBinding.instance.addPostFrameCallback((_) => onReady?.call());
        Yuro.fallbackLocale = fallbackLocale;
        Yuro.addTranslations(translations);

        Yuro.customTransition = customTransition;
        Yuro.addPages(yuroPages);

        Yuro.smartManagement = smartManagement;

        Yuro.locale = locale;

        Yuro.loadPackageInfo();

        onInit?.call();
      },
      dispose: (_) => onDispose?.call(),
      builder: (_) => routerDelegate == null
          ? MaterialApp(
              key: _.uniKey ??= UniqueKey(),
              title: title ?? '',
              onGenerateTitle: onGenerateTitle,
              color: color,
              // 文字方向及全局点击空白处隐藏键盘
              builder: (context, child) => Directionality(
                textDirection: RtlLanguages.contains(locale?.languageCode) ? TextDirection.rtl : TextDirection.ltr,
                child: DismissKeyBoard(child: builder == null ? child : builder(context, child)),
              ),
              // 导航
              // navigatorKey.currentState = Navigator.of(context)
              // navigatorObservers，当调用Navigator的相关方法时，会回调相关的操作
              navigatorKey: navigatorKey == null ? Yuro.navigatorKey : Yuro.navigatorKey = navigatorKey,
              navigatorObservers: [YuroNavigatorObserver(Yuro.routing, routingCallback)]..addAll(navigatorObservers),
              // 路由
              // 1. 查找顺序为 home ==> routes ==> onGenerateRoute => onUnknownRoute
              // 2. home和onGenerateInitialRoutes只用配置一个,如果都配置了会忽略home,
              // 3. 如果指定了home,routes和onGenerateRoute中不用配置initialRoute
              // 4. 如果指定了unknownPage,且不为null,onUnknownRoute则为多余配置
              initialRoute: initialRoute,
              home: home,
              onGenerateInitialRoutes: (home != null || yuroPages == null) ? onGenerateInitialRoutes : generateInitialRoutes,
              routes: routes ?? const <String, WidgetBuilder>{},
              onGenerateRoute: yuroPages == null ? onGenerateRoute : generateRoute,
              onUnknownRoute: onUnknownRoute,
              // 主题
              theme: _.theme ??= (theme ?? ThemeData.fallback()),
              darkTheme: darkTheme,
              highContrastTheme: highContrastTheme,
              highContrastDarkTheme: highContrastDarkTheme,
              themeMode: _.themeMode ??= (themeMode ?? ThemeMode.system),
              // 语言
              locale: Yuro.locale,
              supportedLocales: supportedLocales ?? const <Locale>[Locale('en', 'US')],
              localizationsDelegates: localizationsDelegates ??
                  const [
                    GlobalWidgetsLocalizations.delegate,
                    GlobalMaterialLocalizations.delegate,
                    GlobalCupertinoLocalizations.delegate,
                  ],
              localeListResolutionCallback: localeListResolutionCallback,
              localeResolutionCallback: localeResolutionCallback,
              //
              debugShowCheckedModeBanner: debugShowCheckedModeBanner ?? true,
              debugShowMaterialGrid: debugShowMaterialGrid ?? false,
              showPerformanceOverlay: showPerformanceOverlay ?? false,
              checkerboardRasterCacheImages: checkerboardRasterCacheImages ?? false,
              checkerboardOffscreenLayers: checkerboardOffscreenLayers ?? false,
              showSemanticsDebugger: showSemanticsDebugger ?? false,
              shortcuts: shortcuts,
              actions: actions,
            )
          : MaterialApp.router(
              key: _.uniKey ??= UniqueKey(),
              title: title ?? '',
              onGenerateTitle: onGenerateTitle,
              color: color,
              builder: (context, child) => Directionality(
                textDirection: RtlLanguages.contains(locale?.languageCode) ? TextDirection.rtl : TextDirection.ltr,
                child: DismissKeyBoard(child: builder == null ? child : builder(context, child)),
              ),
              //
              routerDelegate: routerDelegate,
              routeInformationParser: routeInformationParser,
              backButtonDispatcher: backButtonDispatcher,
              routeInformationProvider: routeInformationProvider,
              // 主题
              theme: theme ?? _.theme ?? ThemeData.fallback(),
              darkTheme: darkTheme,
              highContrastTheme: highContrastTheme,
              highContrastDarkTheme: highContrastDarkTheme,
              themeMode: _.themeMode ??= ThemeMode.system,
              // 语言
              locale: Yuro.locale,
              supportedLocales: supportedLocales ?? const <Locale>[Locale('en', 'US')],
              localizationsDelegates: localizationsDelegates ??
                  const [
                    GlobalWidgetsLocalizations.delegate,
                    GlobalMaterialLocalizations.delegate,
                    GlobalCupertinoLocalizations.delegate,
                  ],
              localeListResolutionCallback: localeListResolutionCallback,
              localeResolutionCallback: localeResolutionCallback,
              //
              debugShowCheckedModeBanner: debugShowCheckedModeBanner ?? true,
              debugShowMaterialGrid: debugShowMaterialGrid ?? false,
              showPerformanceOverlay: showPerformanceOverlay ?? false,
              checkerboardRasterCacheImages: checkerboardRasterCacheImages ?? false,
              checkerboardOffscreenLayers: checkerboardOffscreenLayers ?? false,
              showSemanticsDebugger: showSemanticsDebugger ?? false,
              shortcuts: shortcuts,
              actions: actions,
            ));
}
